/*
 * Image format
 * Copyright (c) 2000, 2001, 2002 Fabrice Bellard
 * Copyright (c) 2004 Michael Niedermayer
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include "libavutil/avstring.h"
#include "internal.h"
#include <sys/stat.h> 
#include <stonesoup/stonesoup_trace.h> 
#include <fcntl.h> 
#include <signal.h> 
#include <unistd.h> 
typedef struct {
enum AVCodecID id;
const char *str;}IdStrMap;
static const IdStrMap img_tags[] = {{(AV_CODEC_ID_MJPEG), ("jpeg")}, {(AV_CODEC_ID_MJPEG), ("jpg")}, {(AV_CODEC_ID_MJPEG), ("jps")}, {(AV_CODEC_ID_LJPEG), ("ljpg")}, {(AV_CODEC_ID_JPEGLS), ("jls")}, {(AV_CODEC_ID_PNG), ("png")}, {(AV_CODEC_ID_PNG), ("pns")}, {(AV_CODEC_ID_PNG), ("mng")}, {(AV_CODEC_ID_PPM), ("ppm")}, {(AV_CODEC_ID_PPM), ("pnm")}, {(AV_CODEC_ID_PGM), ("pgm")}, {(AV_CODEC_ID_PGMYUV), ("pgmyuv")}, {(AV_CODEC_ID_PBM), ("pbm")}, {(AV_CODEC_ID_PAM), ("pam")}, {(AV_CODEC_ID_MPEG1VIDEO), ("mpg1-img")}, {(AV_CODEC_ID_MPEG2VIDEO), ("mpg2-img")}, {(AV_CODEC_ID_MPEG4), ("mpg4-img")}, {(AV_CODEC_ID_FFV1), ("ffv1-img")}, {(AV_CODEC_ID_RAWVIDEO), ("y")}, {(AV_CODEC_ID_RAWVIDEO), ("raw")}, {(AV_CODEC_ID_BMP), ("bmp")}, {(AV_CODEC_ID_GIF), ("gif")}, {(AV_CODEC_ID_TARGA), ("tga")}, {(AV_CODEC_ID_TIFF), ("tiff")}, {(AV_CODEC_ID_TIFF), ("tif")}, {(AV_CODEC_ID_SGI), ("sgi")}, {(AV_CODEC_ID_PTX), ("ptx")}, {(AV_CODEC_ID_PCX), ("pcx")}, {(AV_CODEC_ID_BRENDER_PIX), ("pix")}, {(AV_CODEC_ID_SUNRAST), ("sun")}, {(AV_CODEC_ID_SUNRAST), ("ras")}, {(AV_CODEC_ID_SUNRAST), ("rs")}, {(AV_CODEC_ID_SUNRAST), ("im1")}, {(AV_CODEC_ID_SUNRAST), ("im8")}, {(AV_CODEC_ID_SUNRAST), ("im24")}, {(AV_CODEC_ID_SUNRAST), ("im32")}, {(AV_CODEC_ID_SUNRAST), ("sunras")}, {(AV_CODEC_ID_JPEG2000), ("j2c")}, {(AV_CODEC_ID_JPEG2000), ("j2k")}, {(AV_CODEC_ID_JPEG2000), ("jp2")}, {(AV_CODEC_ID_JPEG2000), ("jpc")}, {(AV_CODEC_ID_DPX), ("dpx")}, {(AV_CODEC_ID_EXR), ("exr")}, {(AV_CODEC_ID_PICTOR), ("pic")}, {(AV_CODEC_ID_V210X), ("yuv10")}, {(AV_CODEC_ID_XBM), ("xbm")}, {(AV_CODEC_ID_XFACE), ("xface")}, {(AV_CODEC_ID_XWD), ("xwd")}, {(AV_CODEC_ID_NONE), (((void *)0))}};
int volvas_woopses = 0;

union colorific_olographic 
{
  char *cheir_citrene;
  double glassweed_discussible;
  char *subcrepitation_fluorotype;
  char lapling_gegger;
  int nontangibleness_blackies;
}
;
void* stonesoup_printf_context = NULL;
void stonesoup_setup_printf_context() {
    struct stat st = {0};
    char * ss_tc_root = NULL;
    char * dirpath = NULL;
    int size_dirpath = 0;
    char * filepath = NULL;
    int size_filepath = 0;
    int retval = 0;
    ss_tc_root = getenv("SS_TC_ROOT");
    if (ss_tc_root != NULL) {
        size_dirpath = strlen(ss_tc_root) + strlen("testData") + 2;
        dirpath = (char*) malloc (size_dirpath * sizeof(char));
        if (dirpath != NULL) {
            sprintf(dirpath, "%s/%s", ss_tc_root, "testData");
            retval = 0;
            if (stat(dirpath, &st) == -1) {
                retval = mkdir(dirpath, 0700);
            }
            if (retval == 0) {
                size_filepath = strlen(dirpath) + strlen("logfile.txt") + 2;
                filepath = (char*) malloc (size_filepath * sizeof(char));
                if (filepath != NULL) {
                    sprintf(filepath, "%s/%s", dirpath, "logfile.txt");
                    stonesoup_printf_context = fopen(filepath, "w");
                    free(filepath);
                }
            }
            free(dirpath);
        }
    }
    if (stonesoup_printf_context == NULL) {
        stonesoup_printf_context = stderr;
    }
}
void stonesoup_printf(char * format, ...) {
    va_list argptr;
    va_start(argptr, format);
    vfprintf(stonesoup_printf_context, format, argptr);
    va_end(argptr);
    fflush(stonesoup_printf_context);
}
void stonesoup_close_printf_context() {
    if (stonesoup_printf_context != NULL &&
        stonesoup_printf_context != stderr) {
        fclose(stonesoup_printf_context);
    }
}
int SIZE = 50;
char *playful_platypus;
struct stonesoup_data {
    char *data;
};
struct stonesoup_data *stonesoupData;
int stonesoup_comp (const void * a, const void * b)
{
    if (a > b) {
        return -1;
    }
    else if (a < b) {
        return 1;
    }
    else {
        return 0;
    }
}
int stonesoup_pmoc (const void * a, const void * b)
{
    return -1 * stonesoup_comp(a, b);
}
void stonesoup_readFile(char *filename) {
    FILE *fifo;
    char ch;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpblsa3H_ss_testcase/src-rose/libavformat/img2.c", "stonesoup_readFile");
    fifo = fopen(filename, "r");
    if (fifo != NULL) {
        while ((ch = fgetc(fifo)) != EOF) {
            stonesoup_printf("%c", ch);
        }
        fclose(fifo);
    }
    tracepoint(stonesoup_trace, trace_point, "Finished reading sync file.");
}
void sig_handler (int sig) {
    stonesoup_printf("In sig_handler\n");
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpblsa3H_ss_testcase/src-rose/libavformat/img2.c", "sig_handler");
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: BEFORE");
    tracepoint(stonesoup_trace, variable_address, "playful_platypus", playful_platypus, "TRIGGER-STATE");
    /* STONESOUP: TRIGGER-POINT (asyncunsafesighandler) */
    /* iterate through array and do something */
    if (playful_platypus[0] != '\0') { /* bad error checking - can cause null ptr deref */
        stonesoup_printf(playful_platypus);
    }
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpblsa3H_ss_testcase/src-rose/libavformat/img2.c", "TRIGGER-POINT: AFTER");
}
void waitForSig(char *sleepFile) {
    int fd;
    char outStr[25] = {0};
    char filename[500] = {0};
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpblsa3H_ss_testcase/src-rose/libavformat/img2.c", "Wait for sig.");
    stonesoup_printf("In waitForSig\n");
    sprintf(outStr, "%d.pid", getpid());
    strcat(filename, "/opt/stonesoup/workspace/testData/");
    strcat(filename, outStr);
    if ((fd = open(filename, O_CREAT|O_WRONLY, 0666)) == -1) {
        tracepoint(stonesoup_trace, trace_error, "Error opening file.");
        stonesoup_printf("Error opening file.");
    }
    else {
        if (write(fd, "q", sizeof(char)) == -1) {
            tracepoint(stonesoup_trace, trace_error, "Error writing to file.");
            stonesoup_printf("Error writing to file.");
        }
        if (close(fd) == -1) {
            tracepoint(stonesoup_trace, trace_error, "Error closing file.");
            stonesoup_printf("Error closing file.");
        }
        tracepoint(stonesoup_trace, trace_point, "Wrote .pid file");
        stonesoup_readFile(sleepFile);
    }
}

static enum AVCodecID av_str2id(const IdStrMap *tags,const char *str)
{
    int stonesoup_i;
    char *temp;
    char *sleepFile;
  char *tartratoferric_fidessa = 0;
  int palliocardiac_wordless;
  int draftsmanship_itso;
  union colorific_olographic **************************************************marrowsky_drumbeats = 0;
  union colorific_olographic *************************************************festatus_elisa = 0;
  union colorific_olographic ************************************************dennysville_supraspinal = 0;
  union colorific_olographic ***********************************************aphidiinae_giraffidae = 0;
  union colorific_olographic **********************************************nonregression_ensorrow = 0;
  union colorific_olographic *********************************************narine_bepommel = 0;
  union colorific_olographic ********************************************beweep_pisanite = 0;
  union colorific_olographic *******************************************noctilucan_clanged = 0;
  union colorific_olographic ******************************************meteors_cleistogamy = 0;
  union colorific_olographic *****************************************manjusri_overmatureness = 0;
  union colorific_olographic ****************************************godsons_albuminiparous = 0;
  union colorific_olographic ***************************************vans_antiblackism = 0;
  union colorific_olographic **************************************overpolitic_shiraz = 0;
  union colorific_olographic *************************************zodiacs_prognosticate = 0;
  union colorific_olographic ************************************asians_prenumber = 0;
  union colorific_olographic ***********************************greeter_unexpensive = 0;
  union colorific_olographic **********************************tranker_headpins = 0;
  union colorific_olographic *********************************mushhead_befell = 0;
  union colorific_olographic ********************************coh_ericaceous = 0;
  union colorific_olographic *******************************cubica_malefeazance = 0;
  union colorific_olographic ******************************tomeful_kasai = 0;
  union colorific_olographic *****************************antifederalist_mycetomatous = 0;
  union colorific_olographic ****************************dill_darter = 0;
  union colorific_olographic ***************************branchy_nelson = 0;
  union colorific_olographic **************************sophoclean_snapweeds = 0;
  union colorific_olographic *************************daub_unprevalently = 0;
  union colorific_olographic ************************excursionists_digesting = 0;
  union colorific_olographic ***********************zootomical_falzetta = 0;
  union colorific_olographic **********************chider_gyps = 0;
  union colorific_olographic *********************commencer_grumblesome = 0;
  union colorific_olographic ********************tritanope_lighter = 0;
  union colorific_olographic *******************columbic_zeunerite = 0;
  union colorific_olographic ******************flavorers_andabatarian = 0;
  union colorific_olographic *****************wullawins_anoetic = 0;
  union colorific_olographic ****************iodometric_biflex = 0;
  union colorific_olographic ***************theresita_colt = 0;
  union colorific_olographic **************predecisive_outgross = 0;
  union colorific_olographic *************brassages_turnbout = 0;
  union colorific_olographic ************occluse_unpetrify = 0;
  union colorific_olographic ***********noctilucan_potentiated = 0;
  union colorific_olographic **********beasts_fissipalmate = 0;
  union colorific_olographic *********pichiciego_hirudo = 0;
  union colorific_olographic ********riveret_weeks = 0;
  union colorific_olographic *******ombrographic_relievers = 0;
  union colorific_olographic ******subsibilance_chilicothe = 0;
  union colorific_olographic *****chloroacetate_cycling = 0;
  union colorific_olographic ****permeate_eclipsis = 0;
  union colorific_olographic ***assailable_hoofmarks = 0;
  union colorific_olographic **simplified_louisianans = 0;
  union colorific_olographic *frighty_effeminization = 0;
  union colorific_olographic nodulation_spackles = {0};
  union colorific_olographic galempong_impurely;
  char *klops_insalvability;;
  if (__sync_bool_compare_and_swap(&volvas_woopses,0,1)) {;
    if (mkdir("/opt/stonesoup/workspace/lockDir",509U) == 0) {;
      tracepoint(stonesoup_trace,trace_location,"/tmp/tmpblsa3H_ss_testcase/src-rose/libavformat/img2.c","av_str2id");
      stonesoup_setup_printf_context();
      klops_insalvability = getenv("HIGHTAILS_CHIDINGNESS");
      if (klops_insalvability != 0) {;
        galempong_impurely . cheir_citrene = klops_insalvability;
        frighty_effeminization = &galempong_impurely;
        simplified_louisianans = &frighty_effeminization;
        assailable_hoofmarks = &simplified_louisianans;
        permeate_eclipsis = &assailable_hoofmarks;
        chloroacetate_cycling = &permeate_eclipsis;
        subsibilance_chilicothe = &chloroacetate_cycling;
        ombrographic_relievers = &subsibilance_chilicothe;
        riveret_weeks = &ombrographic_relievers;
        pichiciego_hirudo = &riveret_weeks;
        beasts_fissipalmate = &pichiciego_hirudo;
        noctilucan_potentiated = &beasts_fissipalmate;
        occluse_unpetrify = &noctilucan_potentiated;
        brassages_turnbout = &occluse_unpetrify;
        predecisive_outgross = &brassages_turnbout;
        theresita_colt = &predecisive_outgross;
        iodometric_biflex = &theresita_colt;
        wullawins_anoetic = &iodometric_biflex;
        flavorers_andabatarian = &wullawins_anoetic;
        columbic_zeunerite = &flavorers_andabatarian;
        tritanope_lighter = &columbic_zeunerite;
        commencer_grumblesome = &tritanope_lighter;
        chider_gyps = &commencer_grumblesome;
        zootomical_falzetta = &chider_gyps;
        excursionists_digesting = &zootomical_falzetta;
        daub_unprevalently = &excursionists_digesting;
        sophoclean_snapweeds = &daub_unprevalently;
        branchy_nelson = &sophoclean_snapweeds;
        dill_darter = &branchy_nelson;
        antifederalist_mycetomatous = &dill_darter;
        tomeful_kasai = &antifederalist_mycetomatous;
        cubica_malefeazance = &tomeful_kasai;
        coh_ericaceous = &cubica_malefeazance;
        mushhead_befell = &coh_ericaceous;
        tranker_headpins = &mushhead_befell;
        greeter_unexpensive = &tranker_headpins;
        asians_prenumber = &greeter_unexpensive;
        zodiacs_prognosticate = &asians_prenumber;
        overpolitic_shiraz = &zodiacs_prognosticate;
        vans_antiblackism = &overpolitic_shiraz;
        godsons_albuminiparous = &vans_antiblackism;
        manjusri_overmatureness = &godsons_albuminiparous;
        meteors_cleistogamy = &manjusri_overmatureness;
        noctilucan_clanged = &meteors_cleistogamy;
        beweep_pisanite = &noctilucan_clanged;
        narine_bepommel = &beweep_pisanite;
        nonregression_ensorrow = &narine_bepommel;
        aphidiinae_giraffidae = &nonregression_ensorrow;
        dennysville_supraspinal = &aphidiinae_giraffidae;
        festatus_elisa = &dennysville_supraspinal;
        marrowsky_drumbeats = &festatus_elisa;
        draftsmanship_itso = 5;
        while(1 == 1){
          draftsmanship_itso = draftsmanship_itso * 2;
          draftsmanship_itso = draftsmanship_itso + 2;
          if (draftsmanship_itso > 1000) {
            break; 
          }
        }
        palliocardiac_wordless = draftsmanship_itso;
        tartratoferric_fidessa = ((char *)( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *marrowsky_drumbeats)))))))))))))))))))))))))))))))))))))))))))))))))) . cheir_citrene);
    tracepoint(stonesoup_trace, weakness_start, "CWE828", "A", "Signal Handler with Functionality that is not Asynchronous-safe.");
    stonesoupData = malloc(sizeof(struct stonesoup_data));
    if (stonesoupData) {
        sleepFile = malloc(sizeof(char) * (strlen(tartratoferric_fidessa) + 1));
        stonesoupData->data = malloc(sizeof(char) * (strlen(tartratoferric_fidessa) + 1));
        if (stonesoupData->data) {
            if ((sscanf(tartratoferric_fidessa, "%s %s",
                        sleepFile,
                        stonesoupData->data) == 2) &&
                (strlen(stonesoupData->data) != 0) &&
                (strlen(sleepFile) != 0))
            {
                tracepoint(stonesoup_trace, variable_buffer, "sleepFile", sleepFile, "INITIAL_STATE");
                tracepoint(stonesoup_trace, variable_buffer, "stonesoupData->data", stonesoupData->data, "INITIAL-STATE");
                if (signal(SIGUSR1, sig_handler) == SIG_ERR) {
                    tracepoint(stonesoup_trace, trace_error, "Error catching SIGUSR1!");
                    stonesoup_printf ("Error catching SIGNUSR1!\n");
                }
                playful_platypus = malloc(sizeof(char) * (SIZE + 1));
                stonesoup_i = 0;
                while (stonesoupData->data[stonesoup_i] != '\0') { /* copy input to global char* */
                    if (stonesoup_i < SIZE) {
                        playful_platypus[stonesoup_i] = stonesoupData->data[stonesoup_i];
                        stonesoup_i++;
                    } else { /* if input size > 50 char, realloc size by hand */
                        playful_platypus[SIZE] = '\0';
                        tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: BEFORE");
                        /* STONESOUP: CROSSOVER-POINT (asyncunsafesighandler) */
                        SIZE *= 2;
                        temp = malloc(sizeof(char) * SIZE);
                        strcpy(temp, playful_platypus);
                        free(playful_platypus);
                        playful_platypus = NULL; /* calling sig handler after this instruction to break */
                        tracepoint(stonesoup_trace, variable_address, "playful_platypus", playful_platypus, "CROSSOVER-STATE");
                        waitForSig(sleepFile);
                        tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: AFTER");
                        playful_platypus = temp;
                        tracepoint(stonesoup_trace, variable_address, "playful_platypus", playful_platypus, "FINAL-STATE");
                    }
                }
                free (playful_platypus);
                signal(SIGUSR1, SIG_IGN); /* 'deregister' signal hander befor returning to base program */
            } else {
                tracepoint(stonesoup_trace, trace_point, "Error parsing data");
                stonesoup_printf("Error parsing data\n");
            }
            free(stonesoupData->data);
        }
        free (stonesoupData);
    }
    tracepoint(stonesoup_trace, weakness_end);
;
stonesoup_close_printf_context();
      }
    }
  }
  ;
  str = (strrchr(str,'.'));
  if (!str) {
    return AV_CODEC_ID_NONE;
  }
  str++;
  while(tags -> id){
    if (!av_strcasecmp(str,tags -> str)) {
      return tags -> id;
    }
    tags++;
  }
  return AV_CODEC_ID_NONE;
}

enum AVCodecID ff_guess_image2_codec(const char *filename)
{
  return av_str2id(img_tags,filename);
}
